@@ -541,7 +541,14 @@ class LoggingCmd(Cmd):
 
 
 
-def handle_command(commands, cmd_line_parts=None, program=None, service=None):
+def handle_command(commands, cmd_line_parts=None, program=None, service=None, cmd=None):
+    # Prepare the cmd object to have the args and opts set
+    if cmd is None:
+        cmd = AttributeDict(args=None, opts=None, chain=[])
+    else:
+        cmd.chain.append(AttributeDict(args=cmd.args, opts=cmd.opts))
+	cmd['args'] = None
+        cmd['opts'] = None
     if program is None:
         program = sys.argv[0]
         if os.sep in sys.argv[0]:
@@ -571,7 +578,13 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None):
             commands[None].on_display_help(service, help)
             return 0
         # Need to be smarter here, want to call the inner functions in the scope of the outer.
-        result = commands[None].on_run(service, k, p)
+        cmd['args'] = k
+        cmd['opts'] = p
+        if hasattr(commands[None], 'run'):
+            result = commands[None].run(service, cmd)
+        else:
+            # Backwards compatibility
+            result = commands[None].on_run(service, k, p)
         if result:
             return result
         # Now try to run the sub-command
@@ -582,39 +595,38 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None):
             error = 'No such command `%s\''%sub_command
             sub_command = None
             raise getopt.GetoptError(error)
-        #if main_opts_used:
-        #    new_cmd_line_parts.pop(new_cmd_line_parts.index(sub_command))
         if isinstance(commands[sub_command], dict):
-            return handle_command(commands[sub_command], cmd_line_parts=args, program='%s %s'%(program, sub_command), service=service)
+            return handle_command(commands[sub_command], cmd_line_parts=args, program='%s %s'%(program, sub_command), service=service, cmd=cmd)
         sub_args, sub_opts = process_command(
             args,#new_cmd_line_parts, 
             sub_command, 
             commands[sub_command],
         )
-            #help = assemble_help(
-            #    commands[sub_command], 
-            #    None, 
-            #    '%s %s'%(program, sub_command) 
-            #)
-            #commands[sub_command][None].on_display_help(service, help)
-            #return 0
-        if 1:#else:
-            if not hasattr(commands[sub_command], 'on_initial_convert'):
-                raise Exception('The command for %r has no %r attribute'%(sub_command, 'on_initial_convert'))
-            k, p, h = commands[sub_command].on_initial_convert(
-                service, 
-                commands[sub_command].option_spec, 
-                sub_args, 
-                sub_opts,
+        if not hasattr(commands[sub_command], 'on_initial_convert'):
+            raise Exception('The command for %r has no %r attribute'%(sub_command, 'on_initial_convert'))
+        k, p, h = commands[sub_command].on_initial_convert(
+            service, 
+            commands[sub_command].option_spec, 
+            sub_args, 
+            sub_opts,
+        )
+        if h:
+            help = assemble_help(
+                commands, 
+                sub_command, 
+                program, 
             )
-            if h:
-                help = assemble_help(
-                    commands, 
-                    sub_command, 
-                    program, 
-                )
-                commands[None].on_display_help(service, help)
-                return 0
+            commands[None].on_display_help(service, help)
+            return 0
+        cmd.chain.append(AttributeDict(args=cmd.args, opts=cmd.opts))
+	cmd['args'] = None
+        cmd['opts'] = None
+        cmd['args'] = k
+        cmd['opts'] = p
+        if hasattr(commands[sub_command], 'run'):
+            result = commands[sub_command].run(service, cmd)
+        else:
+            # Backwards compatibility
             result = commands[sub_command].on_run(service, k, p)
     except getopt.GetoptError, err:
         print 'Error:', str(err)
